/*
 
 Copyright (C) 2006 NTT DATA Corporation
 
 This program is free software; you can redistribute it and/or
 Modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation, version 2.
 
 This program is distributed in the hope that it will be
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 */

package com.clustercontrol.jobmanagement.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.commons.util.ConnectionManager;
import com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterBean;
import com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterPK;

/**
 * ジョブ待ち条件マスタのDAOインターフェースを実装するクラス<BR>
 * 
 * @version 2.1.0
 * @since 2.1.0
 * 
 * @see com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterBean
 * @see com.clustercontrol.jobmanagement.dao.JobStartMasterDAO
 */
public class JobStartMasterDAOImpl implements JobStartMasterDAO {
	
	/** ログ出力のインスタンス。 */
	protected static Log m_log = LogFactory.getLog(JobStartMasterDAOImpl.class);
	
	public void init() {
	}
	
	/** 
	 * SELECT文を発行します。
	 * 引数で指定されたプライマリキーで検索し、取得したデータベースの内容をEntity Beanに反映します。
	 * 
	 * @see com.clustercontrol.jobmanagement.dao.JobStartMasterDAO#load(com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterPK, com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterBean)
	 */
	public void load(JobStartMasterPK pk, JobStartMasterBean ejb) throws EJBException {
		m_log.debug("load() start : " + pk.toString());
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet res = null;
		
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();
			
			//SQL文の定義
			String sql = "SELECT * FROM cc_job_start_mst WHERE job_id = ?";
			//SQL文のセット
			stmt = conn.prepareStatement(sql);
			
			stmt.setString(1, pk.getJob_id());
			
			res = stmt.executeQuery();
			
			if (res.next()) {
				//取得した値をBeanにセット
				ejb.setCalendar(res.getInt("calendar"));
				ejb.setCalendar_end_value(res.getInt("calendar_end_value"));
				ejb.setCalendar_id(res.getString("calendar_id"));
				ejb.setCondition_type(res.getInt("condition_type"));
				ejb.setJob_id(res.getString("job_id"));
				ejb.setSkip(res.getInt("skip"));
				ejb.setSkip_end_status(res.getInt("skip_end_status"));
				ejb.setSkip_end_value(res.getInt("skip_end_value"));
				ejb.setSuspend(res.getInt("suspend"));
				ejb.setUnmatch_end_flg(res.getInt("unmatch_end_flg"));
				ejb.setUnmatch_end_status(res.getInt("unmatch_end_status"));
				ejb.setUnmatch_end_value(res.getInt("unmatch_end_value"));
				ejb.setStart_delay(res.getInt("start_delay"));
				ejb.setStart_delay_condition_type(res.getInt("start_delay_condition_type"));
				ejb.setStart_delay_notify(res.getInt("start_delay_notify"));
				ejb.setStart_delay_notify_priority(res.getInt("start_delay_notify_priority"));
				ejb.setStart_delay_operation(res.getInt("start_delay_operation"));
				ejb.setStart_delay_operation_end_value(res.getInt("start_delay_operation_end_value"));
				ejb.setStart_delay_operation_type(res.getInt("start_delay_operation_type"));
				ejb.setStart_delay_session(res.getInt("start_delay_session"));
				ejb.setStart_delay_session_value(res.getInt("start_delay_session_value"));
				ejb.setStart_delay_time(res.getInt("start_delay_time"));
				ejb.setStart_delay_time_value(res.getTime("start_delay_time_value"));
				ejb.setEnd_delay(res.getInt("end_delay"));
				ejb.setEnd_delay_condition_type(res.getInt("end_delay_condition_type"));
				ejb.setEnd_delay_notify(res.getInt("end_delay_notify"));
				ejb.setEnd_delay_notify_priority(res.getInt("end_delay_notify_priority"));
				ejb.setEnd_delay_operation(res.getInt("end_delay_operation"));
				ejb.setEnd_delay_operation_end_value(res.getInt("end_delay_operation_end_value"));
				ejb.setEnd_delay_operation_type(res.getInt("end_delay_operation_type"));
				ejb.setEnd_delay_session(res.getInt("end_delay_session"));
				ejb.setEnd_delay_session_value(res.getInt("end_delay_session_value"));
				ejb.setEnd_delay_job(res.getInt("end_delay_job"));
				ejb.setEnd_delay_job_value(res.getInt("end_delay_job_value"));
				ejb.setEnd_delay_time(res.getInt("end_delay_time"));
				ejb.setEnd_delay_time_value(res.getTime("end_delay_time_value"));
			} else {
				String msg = "JobStartMaster data is not found.";
				m_log.error("load() error : " + pk.toString() + " SQLException " + msg);
				throw new EJBException(msg);
			}
		} catch (SQLException e) {
			m_log.error("load() error : " + pk.toString() + " SQLException");
			throw new EJBException(e.getMessage());
		} finally {
			//コネクション、結果セット、プリペアドステートメントのクローズ
			try {
				if (stmt != null) {
					stmt.close();
				}
				if (res != null) {
					res.close();
				}
				if (conn != null) {
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("load() error : " + pk.toString() + " SQLException " + e1.getMessage());
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("load() end : " + pk.toString());
	}
	
	/**
	 * UPDATE文を発行します。
	 * 引数で指定されたEntity Beanの内容でデータベースを更新します。
	 * 
	 * @see com.clustercontrol.jobmanagement.dao.JobStartMasterDAO#store(com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterBean)
	 */
	public void store(JobStartMasterBean ejb) throws EJBException {
		m_log.debug("store() start : " + ejb.getJob_id());
		
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();
			//SQL文の定義
			StringBuilder sql = new StringBuilder();
			sql.append("UPDATE cc_job_start_mst SET ");
			sql.append("condition_type = ?, ");
			sql.append("suspend = ?, ");
			sql.append("skip = ?, ");
			sql.append("skip_end_status = ?, ");
			sql.append("skip_end_value = ?, ");
			sql.append("unmatch_end_flg = ?, ");
			sql.append("unmatch_end_status = ?, ");
			sql.append("unmatch_end_value = ?, ");
			sql.append("calendar = ?, ");
			sql.append("calendar_id = ?, ");
			sql.append("calendar_end_value = ? ");
			sql.append("start_delay = ?, ");
			sql.append("start_delay_session = ?, ");
			sql.append("start_delay_session_value = ?, ");
			sql.append("start_delay_time = ?, ");
			sql.append("start_delay_time_value = ?, ");
			sql.append("start_delay_condition_type = ?, ");
			sql.append("start_delay_notify = ?, ");
			sql.append("start_delay_notify_priority = ?, ");
			sql.append("start_delay_operation = ?, ");
			sql.append("start_delay_operation_type = ?, ");
			sql.append("start_delay_operation_end_value = ?, ");
			sql.append("end_delay = ?, ");
			sql.append("end_delay_session = ?, ");
			sql.append("end_delay_session_value = ?, ");
			sql.append("end_delay_job = ?, ");
			sql.append("end_delay_job_value = ?, ");
			sql.append("end_delay_time = ?, ");
			sql.append("end_delay_time_value = ?, ");
			sql.append("end_delay_condition_type = ?, ");
			sql.append("end_delay_notify = ?, ");
			sql.append("end_delay_notify_priority = ?, ");
			sql.append("end_delay_operation = ?, ");
			sql.append("end_delay_operation_type = ?, ");
			sql.append("end_delay_operation_end_value = ? ");
			sql.append("WHERE job_id = ?");
			
			//SQL文のセット
			stmt = conn.prepareStatement(sql.toString());
			
			stmt.setInt(1, ejb.getCondition_type());
			stmt.setInt(2, ejb.getSuspend());
			stmt.setInt(3, ejb.getSkip());
			stmt.setInt(4, ejb.getSkip_end_status());
			stmt.setInt(5, ejb.getSkip_end_value());
			stmt.setInt(6, ejb.getUnmatch_end_flg());
			stmt.setInt(7, ejb.getUnmatch_end_status());
			stmt.setInt(8, ejb.getUnmatch_end_value());
			stmt.setInt(9, ejb.getCalendar());
			stmt.setString(10, ejb.getCalendar_id());
			stmt.setInt(11, ejb.getCalendar_end_value());
			stmt.setInt(12, ejb.getStart_delay());
			stmt.setInt(13, ejb.getStart_delay_session());
			stmt.setInt(14, ejb.getStart_delay_session_value());
			stmt.setInt(15, ejb.getStart_delay_time());
			stmt.setTime(16, ejb.getStart_delay_time_value());
			stmt.setInt(17, ejb.getStart_delay_condition_type());
			stmt.setInt(18, ejb.getStart_delay_notify());
			stmt.setInt(19, ejb.getStart_delay_notify_priority());
			stmt.setInt(20, ejb.getStart_delay_operation());
			stmt.setInt(21, ejb.getStart_delay_operation_type());
			stmt.setInt(22, ejb.getStart_delay_operation_end_value());
			stmt.setInt(23, ejb.getEnd_delay());
			stmt.setInt(24, ejb.getEnd_delay_session());
			stmt.setInt(25, ejb.getEnd_delay_session_value());
			stmt.setInt(26, ejb.getEnd_delay_job());
			stmt.setInt(27, ejb.getEnd_delay_job_value());
			stmt.setInt(28, ejb.getEnd_delay_time());
			stmt.setTime(39, ejb.getEnd_delay_time_value());
			stmt.setInt(30, ejb.getEnd_delay_condition_type());
			stmt.setInt(31, ejb.getEnd_delay_notify());
			stmt.setInt(32, ejb.getEnd_delay_notify_priority());
			stmt.setInt(33, ejb.getEnd_delay_operation());
			stmt.setInt(34, ejb.getEnd_delay_operation_type());
			stmt.setInt(35, ejb.getEnd_delay_operation_end_value());
			stmt.setString(36, ejb.getJob_id());
			
			int row = stmt.executeUpdate();
			
			if (row != 1) {
				String msg = "result row is not 1";
				m_log.error("store() error : " + ejb.getJob_id() + " SQLException" + msg);
				throw new EJBException(msg);
			}
		} catch (SQLException e) {
			m_log.error("store() error : " + ejb.getJob_id() + " SQLException");
			throw new EJBException(e.getMessage());
		} finally {
			// コネクション、結果セット、プリペアドステートメントのクローズ
			try {
				if (stmt != null) {
					stmt.close();
				}
				if (conn != null) {
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("store() error : " + ejb.getJob_id() + " SQLException");
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("store() end : " + ejb.getJob_id());
	}
	
	/**
	 * DELETE文を発行します。
	 * 引数で指定されたプライマリキーでデータベースから削除します。
	 * 
	 * @see com.clustercontrol.jobmanagement.dao.JobStartMasterDAO#remove(com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterPK)
	 */
	public void remove(JobStartMasterPK pk) throws RemoveException, EJBException {
		m_log.debug("remove() start : " + pk.toString());
		
		Connection conn = null;
		PreparedStatement stmt = null;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();
			
			//SQL文の定義
			String sql = "DELETE FROM cc_job_start_mst WHERE job_id = ?";
			//SQL文のセット
			stmt = conn.prepareStatement(sql);
			
			stmt.setString(1, pk.getJob_id());
			
			int row = stmt.executeUpdate();
			
			if (row != 1 ) {
				String msg = "result row is not 1";
				m_log.error("remove() error : " + pk.toString() + " SQLException" + msg);
				throw new EJBException(msg);	
			}
		} catch (SQLException e) {
			m_log.error("remove() error : " + pk.toString() + " SQLException");
			throw new EJBException(e.getMessage());
			
		} finally {
			try {
				if(stmt != null){
					stmt.close();
				}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("remove() error : " + pk.toString() + " SQLException");
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("remove() end : " + pk.toString());
	}
	
	/**
	 * INSERT文を発行します。
	 * 引数で指定されたEntity Beanの内容をデータベースに挿入し、プライマリキーを返します。
	 * 
	 * @see com.clustercontrol.jobmanagement.dao.JobStartMasterDAO#create(com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterBean)
	 */
	public JobStartMasterPK create(JobStartMasterBean ejb) throws CreateException, EJBException {
		m_log.debug("create() start : " + ejb.getJob_id());
		
		Connection conn = null;
		PreparedStatement stmt = null;
		JobStartMasterPK pk;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();
			//SQL文の定義
			StringBuilder sql = new StringBuilder();
			sql.append("INSERT INTO cc_job_start_mst (");
			sql.append("job_id, ");
			sql.append("condition_type, ");
			sql.append("suspend, ");
			sql.append("skip, ");
			sql.append("skip_end_status, ");
			sql.append("skip_end_value, ");
			sql.append("unmatch_end_flg, ");
			sql.append("unmatch_end_status, ");
			sql.append("unmatch_end_value, ");
			sql.append("calendar, ");
			sql.append("calendar_id, ");
			sql.append("calendar_end_value, ");
			sql.append("start_delay, ");
			sql.append("start_delay_session, ");
			sql.append("start_delay_session_value, ");
			sql.append("start_delay_time, ");
			sql.append("start_delay_time_value, ");
			sql.append("start_delay_condition_type, ");
			sql.append("start_delay_notify, ");
			sql.append("start_delay_notify_priority, ");
			sql.append("start_delay_operation, ");
			sql.append("start_delay_operation_type, ");
			sql.append("start_delay_operation_end_value, ");
			sql.append("end_delay, ");
			sql.append("end_delay_session, ");
			sql.append("end_delay_session_value, ");
			sql.append("end_delay_job, ");
			sql.append("end_delay_job_value, ");
			sql.append("end_delay_time, ");
			sql.append("end_delay_time_value, ");
			sql.append("end_delay_condition_type, ");
			sql.append("end_delay_notify, ");
			sql.append("end_delay_notify_priority, ");
			sql.append("end_delay_operation, ");
			sql.append("end_delay_operation_type, ");
			sql.append("end_delay_operation_end_value) ");
			sql.append("VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
			
			//SQL文のセット
			stmt = conn.prepareStatement(sql.toString());
			
			stmt.setString(1, ejb.getJob_id());
			stmt.setInt(2, ejb.getCondition_type());
			stmt.setInt(3, ejb.getSuspend());
			stmt.setInt(4, ejb.getSkip());
			stmt.setInt(5, ejb.getSkip_end_status());
			stmt.setInt(6, ejb.getSkip_end_value());
			stmt.setInt(7, ejb.getUnmatch_end_flg());
			stmt.setInt(8, ejb.getUnmatch_end_status());
			stmt.setInt(9, ejb.getUnmatch_end_value());
			stmt.setInt(10, ejb.getCalendar());
			stmt.setString(11, ejb.getCalendar_id());
			stmt.setInt(12, ejb.getCalendar_end_value());
			stmt.setInt(13, ejb.getStart_delay());
			stmt.setInt(14, ejb.getStart_delay_session());
			stmt.setInt(15, ejb.getStart_delay_session_value());
			stmt.setInt(16, ejb.getStart_delay_time());
			stmt.setTime(17, ejb.getStart_delay_time_value());
			stmt.setInt(18, ejb.getStart_delay_condition_type());
			stmt.setInt(19, ejb.getStart_delay_notify());
			stmt.setInt(20, ejb.getStart_delay_notify_priority());
			stmt.setInt(21, ejb.getStart_delay_operation());
			stmt.setInt(22, ejb.getStart_delay_operation_type());
			stmt.setInt(23, ejb.getStart_delay_operation_end_value());
			stmt.setInt(24, ejb.getEnd_delay());
			stmt.setInt(25, ejb.getEnd_delay_session());
			stmt.setInt(26, ejb.getEnd_delay_session_value());
			stmt.setInt(27, ejb.getEnd_delay_job());
			stmt.setInt(28, ejb.getEnd_delay_job_value());
			stmt.setInt(29, ejb.getEnd_delay_time());
			stmt.setTime(30, ejb.getEnd_delay_time_value());
			stmt.setInt(31, ejb.getEnd_delay_condition_type());
			stmt.setInt(32, ejb.getEnd_delay_notify());
			stmt.setInt(33, ejb.getEnd_delay_notify_priority());
			stmt.setInt(34, ejb.getEnd_delay_operation());
			stmt.setInt(35, ejb.getEnd_delay_operation_type());
			stmt.setInt(36, ejb.getEnd_delay_operation_end_value());
			
			int row =	stmt.executeUpdate();
			pk = new JobStartMasterPK(ejb.getJob_id());
			
			if (row != 1) {
				String msg = "result row is not 1";
				m_log.error("create() error : " + ejb.getJob_id() + " SQLException");
				throw new EJBException(msg);
			}
		} catch (SQLException e) {
			if(e.getSQLState().equals("23505")){
				m_log.error("create() error : " + ejb.getJob_id() + " SQLException" + "DuplicateKeyException ");
				throw new DuplicateKeyException(e.getMessage());
			}
			throw new CreateException(e.getMessage());
		}finally{
			try {
				if(stmt != null){
					stmt.close();
				}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("create() error : " + ejb.getJob_id() + " SQLException");
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("create() end : " + ejb.getJob_id());
		return pk;
	}
	
	/**
	 * 全件取得するSELECT文を発行します。
	 * 取得したデータのプライマリキーをコレクションに格納し返します。
	 * 
	 * @see com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterBean#ejbFindAll()
	 * @see com.clustercontrol.jobmanagement.dao.JobStartMasterDAO#findAll()
	 */
	public Collection findAll() throws FinderException {
		m_log.debug("findAll() start : ");
		
		ArrayList<JobStartMasterPK> ret = new ArrayList<JobStartMasterPK>();
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet res = null;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();	
			//SQL文の定義
			String sql = "SELECT job_id FROM cc_job_start_mst";
			//SQL文のセット
			stmt = conn.prepareStatement(sql);
			res = stmt.executeQuery();
			
			while(res.next()) {
				JobStartMasterPK pk= new JobStartMasterPK(
						res.getString("job_id")
				);
				ret.add(pk);
			}
			
		} catch (SQLException e) {
			m_log.error("findAll() error :  SQLException");
			throw new EJBException(e.getMessage());
		}  finally{
			try {
				if(stmt != null){
					stmt.close();}
				if(res != null){
					res.close();}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("findAll() error :  SQLException");
				throw new EJBException(e1.getMessage());
			}
		}
		m_log.debug("findAll() end : ");
		return ret;
	}
	
	/**
	 * 1件取得するSELECT文を発行します。
	 * 引数で指定されたプライマリキーで、データベースを検索します。
	 * 
	 * @see com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterBean#ejbFindByPrimaryKey(com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterPK)
	 * @see com.clustercontrol.jobmanagement.dao.JobStartMasterDAO#findByPrimaryKey(com.clustercontrol.jobmanagement.ejb.entity.JobStartMasterPK)
	 */
	public JobStartMasterPK findByPrimaryKey(JobStartMasterPK pk) throws FinderException {
		m_log.debug("findByPrimaryKey() start : " + pk.toString());
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet res = null;
		try {
			conn = ConnectionManager.getConnectionManager().getConnection();
			
			//SQL文の定義
			String sql = "SELECT job_id FROM cc_job_start_mst WHERE job_id = ?";
			//SQL文のセット
			stmt = conn.prepareStatement(sql);
			
			stmt.setString(1, pk.getJob_id());
			
			res = stmt.executeQuery();
			
			if (res.next()) {
				return pk;
			} else {
				String msg = "id " + pk.toString() + " not found.";
				throw new FinderException(msg);
			}
		} catch (SQLException e) {
			m_log.error("find() error : " + pk.toString() + " SQLException");
			throw new EJBException(e.getMessage());
		}  finally{
			try {
				if(stmt != null){
					stmt.close();}
				if(res != null){
					res.close();}
				if(conn != null){
					conn.close();
				}
			} catch (SQLException e1) {
				m_log.error("find() error : " + pk.toString() + " SQLException");
				throw new EJBException(e1.getMessage());
			}
			m_log.debug("findByPrimaryKey() end : " + pk.toString());
		}
	}
}
